﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using UnityEngine;
namespace Qiyu.Sdk.Tool
{
    public static class MRCUtil
    {
        public static Vector3 TransformPoint(Vector3 originPos, Quaternion originRot, Vector3 originScale, Vector3 localPos)
        {
            return originRot * Vector3.Scale(localPos, originScale) + originPos;
        }

        public static Vector3 FromVector3f(this Vector3f v)
        {
            return new Vector3() { x = v.x, y = v.y, z = v.z };
        }
        public static Quaternion FromQuatf(this Quatf q)
        {
            return new Quaternion() { x = q.x, y = q.y, z = q.z, w = q.w };
        }


        /// <summary>
        /// OnMRCInfoChanged
        /// </summary>
        public static Action<float, Vector3, Vector3> OnMRCInfoChanged;

        [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
        public delegate void InfoChangedCallBack(float fov, float posX, float posY,
                                          float posZ, float rotX, float rotY, float rotZ);
        public static IntPtr OnInfoChangedCallBack
        {
            get
            {
                InfoChangedCallBack callback_delegate = OnInfoChangedNative;
                return Marshal.GetFunctionPointerForDelegate(callback_delegate);
            }
        }

        [AOT.MonoPInvokeCallback(typeof(InfoChangedCallBack))]
        public static void OnInfoChangedNative(float fov, float posX, float posY,
                                          float posZ, float rotX, float rotY, float rotZ)
        {
            try
            {
                if (OnMRCInfoChanged != null)
                {
                    OnMRCInfoChanged(fov, new Vector3(posX, posY, posZ), new Vector3(rotX, rotY, rotZ));
                }
            }
            catch (Exception e)
            {
                Debug.LogError(e.Message);
            }
        }

        /// <summary>
        /// OnRecenter
        /// </summary>
        public static Action OnRecenter;

        [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
        public delegate void RecenterCallBack();
        public static IntPtr OnRecenterCallBack
        {
            get
            {
                RecenterCallBack cb = OnRecenterNative;
                return Marshal.GetFunctionPointerForDelegate(cb);
            }
        }

        [AOT.MonoPInvokeCallback(typeof(RecenterCallBack))]
        public static void OnRecenterNative()
        {
            try
            {
                if (OnRecenter != null)
                {
                    OnRecenter();
                }
            }
            catch (Exception e)
            {
                Debug.LogError(e.Message);
            }
        }
    }
}