﻿//=============================================================================
//
//          Copyright (c) 2022 Beijing Dreamagic Science and Technology Co.,Ltd.
//                          All Rights Reserved.
//
//=============================================================================
using System;
using System.Collections;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using UnityEngine;
using UnityEngine.XR;

namespace Qiyu.Sdk.Tool
{
    public class QiyuMRC : MonoBehaviour
    {
        enum RenderEventType
        {
            InitMRCRender,
            UpdateMRCRender
        };

        private RenderTexture rt_back = null;
        private RenderTexture rt_front = null;
        [HideInInspector]
        public Camera cam_back;
        [HideInInspector]
        public Camera cam_front;

        [QiyuInspectorName("Only Background")]
        public bool onlyBack = false;
        [QiyuInspectorName("Foreground Camera Layers")]
        public LayerMask frontLayerMask = -1;
        [QiyuInspectorName("Background Camera Layers")]
        public LayerMask backLayerMask = -1;
        [HideInInspector]
        public Color32 cullColor = new Color32();

        // Start is called before the first frame update
        IEnumerator Start()
        {
            cam_back = this.transform.Find("Cam_Back").GetComponent<Camera>();
            cam_back.cullingMask = backLayerMask;
            cam_front = this.transform.Find("Cam_Front").GetComponent<Camera>();
            cam_front.cullingMask = backLayerMask;
            if (onlyBack)
            {
                cam_front.gameObject.SetActive(false);
            }

            if (QiyuPlatform.IsAndroid)
            {
                QiyuMRCPlugin.GetFrontCullColor(ref cullColor);
                cam_front.backgroundColor = cullColor;
                Debug.LogFormat("Use CullColor:{0}", cullColor);

                Vector2f size = new Vector2f();
                QiyuMRCPlugin.GetRenderSize(ref size);
                rt_back = RenderTexture.GetTemporary((int)size.x, (int)size.y, 24, RenderTextureFormat.Default, RenderTextureReadWrite.Default);
                rt_back.name = "rt_back";
                cam_back.targetTexture = rt_back;
                if (!onlyBack)
                {
                    rt_front = RenderTexture.GetTemporary((int)size.x, (int)size.y, 24, RenderTextureFormat.Default, RenderTextureReadWrite.Default);
                    rt_front.name = "rt_back";
                    cam_front.targetTexture = rt_front;
                }
                //设置FOV
                cam_back.fieldOfView = QiyuMRCPlugin.GetCameraFov();
                if (!onlyBack)
                {
                    cam_front.fieldOfView = QiyuMRCPlugin.GetCameraFov();
                }

                //设置相机位置
                ReadCameraPos();

                //设置纹理
                QiyuMRCPlugin.SetCameraTexture(rt_back.GetNativeTexturePtr(), onlyBack ? IntPtr.Zero : rt_front.GetNativeTexturePtr(), QualitySettings.activeColorSpace == ColorSpace.Linear ? 1 : 0);
                GL.IssuePluginEvent(QiyuMRCPlugin.GetRenderEventFunc(), (int)RenderEventType.InitMRCRender);

                //启动更新
                StartCoroutine("OnNativeUpdate");

                MRCUtil.OnRecenter += OnRecenter;
            }
            else
            {
                //这里防止编辑器运行遮挡主相机
                cam_back.depth = -1;
                cam_front.depth = -1;
            }
            yield return null;
        }

        private void ReadCameraPos()
        {
            Vector3f pos = new Vector3f();
            Quatf rot = Quatf.identity;
            if (QiyuPlatform.IsAndroid)
            {
                QiyuMRCPlugin.GetCameraPose(ref pos, ref rot);
            }
            this.transform.localPosition = pos.FromVector3f();
            this.transform.localRotation = rot.FromQuatf();
        }

        private void OnDestroy()
        {
            MRCUtil.OnRecenter -= OnRecenter;
            if (rt_back != null)
            {
                RenderTexture.ReleaseTemporary(rt_back);
            }
            if (rt_front != null)
            {
                RenderTexture.ReleaseTemporary(rt_front);
            }
        }

        private void OnApplicationPause(bool pause)
        {
            if (QiyuPlatform.IsAndroid)
            {
                QiyuMRCPlugin.OnMRCPause(pause);
            }
        }

        IEnumerator OnNativeUpdate()
        {
            while (true)
            {
                // Wait until all frame rendering is done
                yield return new WaitForEndOfFrame();

                //设置相机裁减面
                float dis = Vector3.Distance(Camera.main.transform.position, cam_front.transform.position);
                //Debug.Log("Update farClipPlane:" + cam_front.farClipPlane);

                if (!onlyBack)
                {
                    //设置front远裁减面
                    cam_front.farClipPlane = dis;
                    cam_back.nearClipPlane = dis;
                }

                if (QiyuPlatform.IsAndroid)
                {
                    QiyuMRCPlugin.OnMRCUpdate(Time.deltaTime);
                    GL.IssuePluginEvent(QiyuMRCPlugin.GetRenderEventFunc(), (int)RenderEventType.UpdateMRCRender);
                }
            }
        }

        private void OnRecenter()
        {
            ReadCameraPos();
        }
    }
}