﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using UnityEngine;

namespace Qiyu.Sdk.Tool
{
    public class QiyuMRCPlugin
    {
        [RuntimeInitializeOnLoadMethod(RuntimeInitializeLoadType.BeforeSceneLoad)]
        public static void AutoInit()
        {
            if (QiyuPlatform.IsAndroid)
            {
                //初始化
                SetupMRCTool(IntPtr.Zero);

                //设置微调回调
                SetMRCInfoChangedCallBack(MRCUtil.OnInfoChangedCallBack);

                //设置Recenter回调
                SetOnRecenterCallBack(MRCUtil.OnRecenterCallBack);
            }
        }

        public const string pluginName = "QiyuMRCTool";

        [DllImport(pluginName, CallingConvention = CallingConvention.Cdecl)]
        public static extern void SetupMRCTool(IntPtr activity);

        [DllImport(pluginName, CallingConvention = CallingConvention.Cdecl)]
        public static extern void SetCameraTexture(System.IntPtr back, System.IntPtr front, int colorSpace);

        [DllImport(pluginName, CallingConvention = CallingConvention.Cdecl)]
        public static extern void GetRenderSize(ref Vector2f size);

        [DllImport(pluginName, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr GetRenderEventFunc();

        [DllImport(pluginName, CallingConvention = CallingConvention.Cdecl)]
        public static extern void GetCameraPose(ref Vector3f pos, ref Quatf rot, int platform = 1);

        [DllImport(pluginName, CallingConvention = CallingConvention.Cdecl)]
        public static extern float GetCameraFov();

        [DllImport(pluginName, CallingConvention = CallingConvention.Cdecl)]
        public static extern void OnMRCPause(bool b);

        [DllImport(pluginName, CallingConvention = CallingConvention.Cdecl)]
        public static extern void OnMRCUpdate(float t);

        [DllImport(pluginName, CallingConvention = CallingConvention.Cdecl)]
        public static extern void SetMRCInfoChangedCallBack(IntPtr callBack);

        [DllImport(pluginName, CallingConvention = CallingConvention.Cdecl)]
        public static extern void SetOnRecenterCallBack(IntPtr callBack);

        [DllImport(pluginName, CallingConvention = CallingConvention.Cdecl)]
        public static extern void GetFrontCullColor(ref Color32 color);
    }


    [StructLayout(LayoutKind.Sequential)]
    public struct Vector2f
    {
        public float x;
        public float y;
        public static readonly Vector3f zero = new Vector3f { x = 0.0f, y = 0.0f };
        public override string ToString()
        {
            return string.Format("{0}, {1}", x, y);
        }
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct Vector3f
    {
        public float x;
        public float y;
        public float z;
        public static readonly Vector3f zero = new Vector3f { x = 0.0f, y = 0.0f, z = 0.0f };
        public override string ToString()
        {
            return string.Format("{0}, {1}, {2}", x, y, z);
        }
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct Quatf
    {
        public float x;
        public float y;
        public float z;
        public float w;
        public static readonly Quatf identity = new Quatf { x = 0.0f, y = 0.0f, z = 0.0f, w = 1.0f };
        public override string ToString()
        {
            return string.Format("{0}, {1}, {2}, {3}", x, y, z, w);
        }
    }


    public enum Bool
    {
        False = 0,
        True
    }
}